@@ -11,6 +11,7 @@ r::List Multiview_rcpp ( std::string  pathIn,
                          std::string  predictFile,
                          std::string  lib,
                          std::string  pred,
+                         int          D,
                          int          E,
                          int          Tp,
                          int          knn,
@@ -19,6 +20,7 @@ r::List Multiview_rcpp ( std::string  pathIn,
                          std::string  target,
                          int          multiview,
                          int          exclusionRadius,
+                         bool         trainLib,
                          bool         verbose,
                          unsigned int numThreads ) {
 
@@ -32,7 +34,8 @@ r::List Multiview_rcpp ( std::string  pathIn,
                         pathOut,
                         predictFile,
                         lib,
-                        pred, 
+                        pred,
+                        D,
                         E,
                         Tp,
                         knn,
@@ -41,6 +44,7 @@ r::List Multiview_rcpp ( std::string  pathIn,
                         target,
                         multiview,
                         exclusionRadius,
+                        trainLib,
                         verbose,
                         numThreads );
     }
@@ -51,7 +55,8 @@ r::List Multiview_rcpp ( std::string  pathIn,
                         pathOut,
                         predictFile,
                         lib,
-                        pred, 
+                        pred,
+                        D,
                         E,
                         Tp,
                         knn,
@@ -60,6 +65,7 @@ r::List Multiview_rcpp ( std::string  pathIn,
                         target,
                         multiview,
                         exclusionRadius,
+                        trainLib,
                         verbose,
                         numThreads );
     }
