@@ -1,3 +1,7 @@
+
+// R to C++ interface using Rcpp
+// Functional flow: R func calls Rcpp func calls C++ func.
+
 #ifndef RCPPEDMCOMMON
 #define RCPPEDMCOMMON
 
@@ -59,6 +63,7 @@ r::List Multiview_rcpp ( std::string  pathIn,
                          std::string  predictFile,
                          std::string  lib,
                          std::string  pred,
+                         int          D,
                          int          E,
                          int          Tp,
                          int          knn,
@@ -67,6 +72,7 @@ r::List Multiview_rcpp ( std::string  pathIn,
                          std::string  target,
                          int          multiview,
                          int          exlcusionRadius,
+                         bool         trainLib,
                          bool         verbose,
                          unsigned int numThreads );
 
@@ -103,23 +109,24 @@ r::DataFrame MakeBlock_rcpp( r::DataFrame             dataList,
 r::List ComputeError_rcpp ( std::vector<double> vec1, 
                             std::vector<double> vec2 );
 
-r::DataFrame CCM_rcpp( std::string  pathIn,
-                       std::string  dataFile,
-                       r::DataFrame dataList,
-                       std::string  pathOut,
-                       std::string  predictFile,
-                       int          E,
-                       int          Tp,
-                       int          knn,
-                       int          tau, 
-                       std::string  columns,
-                       std::string  target,
-                       std::string  libSizes,
-                       int          sample,
-                       bool         random,
-                       bool         replacement,
-                       unsigned     seed, 
-                       bool         verbose );
+r::List CCM_rcpp( std::string  pathIn,
+                  std::string  dataFile,
+                  r::DataFrame dataList,
+                  std::string  pathOut,
+                  std::string  predictFile,
+                  int          E,
+                  int          Tp,
+                  int          knn,
+                  int          tau, 
+                  std::string  columns,
+                  std::string  target,
+                  std::string  libSizes,
+                  int          sample,
+                  bool         random,
+                  bool         replacement,
+                  unsigned     seed,
+                  bool         includeData,
+                  bool         verbose );
 
 r::DataFrame Simplex_rcpp( std::string  pathIn,
                            std::string  dataFile,
