@@ -3,23 +3,32 @@
 // Data input requires columns to specify timeseries columns
 // that will be embedded by Embed(), and target for predictions.
 // 
-// E represents the number of variables to combine for each
-// assessment, as well as the number of time delays to create in 
-// Embed() for each variable.
+// D represents the number of variables to combine for each
+// assessment, if not specified, it is the number of columns.
+// E is the embedding dimension of each variable. 
+// If E = 1, no time delay embedding is done, but the variables
+// in the embedding are named X(t-0), Y(t-0)...
 // 
-// multiview is the number of top-ranked E-dimensional predictions
+// Parameters.Validate() with Method::Simplex sets knn equal to E+1
+// if knn not specified, so we need to explicitly set knn to D + 1.
+// 
+// multiview is the number of top-ranked D-dimensional predictions
 // to "average" for the final prediction. Corresponds to parameter
 // k in Ye & Sugihara with default k = sqrt(m) where m is the
-// number of combinations C(n,E) available from the n = nColumns * E
-// columns taken E at-a-time. 
+// number of combinations C(n,D) available from the n = D * E
+// columns taken D at-a-time. 
 //
-// Parameters.Validate() with Method::Simplex sets knn equal to E+1
-// if knn not specified.
-// 
 // Ye H., and G. Sugihara, 2016. Information leverage in
 // interconnected ecosystems: Overcoming the curse of dimensionality.
 // Science 353:922–925.
 //--------------------------------------------------------------------
+//
+// NOTE: Multiview evaluates the top projections using in-sample
+//       library predictions.  It can be shown that highly accurate
+//       in-sample predictions can be made from arbitrary non-
+//       constant, non-oscillatory vectors. Therefore, some attention
+//       may be warranted to filter prospective embedding vectors.
+//--------------------------------------------------------------------
 
 #include <thread>
 #include <atomic>
@@ -74,6 +83,7 @@ MultiviewValues Multiview( std::string pathIn,
                            std::string predictFile,
                            std::string lib,
                            std::string pred,
+                           int         D,
                            int         E,
                            int         Tp,
                            int         knn,
@@ -82,6 +92,7 @@ MultiviewValues Multiview( std::string pathIn,
                            std::string target,
                            int         multiview,
                            int         exclusionRadius,
+                           bool        trainLib,
                            bool        verbose,
                            unsigned    nThreads ) {
 
@@ -93,6 +104,7 @@ MultiviewValues Multiview( std::string pathIn,
                                         predictFile,
                                         lib,
                                         pred,
+                                        D,
                                         E,
                                         Tp,
                                         knn,
@@ -101,6 +113,7 @@ MultiviewValues Multiview( std::string pathIn,
                                         target,
                                         multiview,
                                         exclusionRadius,
+                                        trainLib,
                                         verbose,
                                         nThreads );
     return result;
@@ -110,28 +123,38 @@ MultiviewValues Multiview( std::string pathIn,
 // Multiview()
 // API Overload 2: DataFrame provided
 //----------------------------------------------------------------
-MultiviewValues  Multiview( DataFrame< double > data,
-                            std::string         pathOut,
-                            std::string         predictFile,
-                            std::string         lib,
-                            std::string         pred,
-                            int                 E,
-                            int                 Tp,
-                            int                 knn,
-                            int                 tau,
-                            std::string         columns,
-                            std::string         target,
-                            int                 multiview,
-                            int                 exclusionRadius,
-                            bool                verbose,
-                            unsigned            nThreads ) {
+MultiviewValues Multiview( DataFrame< double > data,
+                           std::string         pathOut,
+                           std::string         predictFile,
+                           std::string         lib,
+                           std::string         pred,
+                           int                 D,
+                           int                 E,
+                           int                 Tp,
+                           int                 knn,
+                           int                 tau,
+                           std::string         columns,
+                           std::string         target,
+                           int                 multiview,
+                           int                 exclusionRadius,
+                           bool                trainLib,
+                           bool                verbose,
+                           unsigned            nThreads ) {
 
+    // Require at least E = 1
+    if ( E < 1 ) {
+        std::stringstream errMsg;
+        errMsg << " Multiview(): E = " << E << " is invalid.\n" ;
+        throw std::runtime_error( errMsg.str() );
+    }
+    
     // Create local Parameters struct. Note embedded = true
     Parameters param = Parameters( Method::Simplex, "", "",
                                    pathOut, predictFile,
                                    lib, pred, E, Tp, knn, tau, 0,
                                    exclusionRadius, columns, target,
-                                   true, false, verbose,
+                                   true, // embedded true
+                                   false, verbose,
                                    "", "", "", 0, 0, 0, multiview );
 
     if ( not param.columnNames.size() ) {
@@ -140,9 +163,6 @@ MultiviewValues  Multiview( DataFrame< double > data,
     if ( not param.targetName.size() ) {
         throw std::runtime_error( "Multiview() requires target name." );
     }
-    if ( param.E < 1 ) {
-        throw std::runtime_error( "Multiview() E is required." );
-    }
     // Ensure that params are validated so columnNames are populated
     if ( not param.validated ) {
         throw std::runtime_error( "Multiview() params not validated." );        
@@ -180,7 +200,9 @@ MultiviewValues  Multiview( DataFrame< double > data,
 
     //------------------------------------------------------------
     // Generate embedding on param.columns_str
-    // embedding will have tau * (E-1) fewer rows than data
+    // embedding will have tau * (E-1) fewer rows than data,
+    // the time column is not returned in embedding.
+    // Column names are changed from X to X(t-0), X(t-1),...
     //------------------------------------------------------------
     DataFrame< double > embedding = Embed( data,
                                            param.E,
@@ -188,11 +210,6 @@ MultiviewValues  Multiview( DataFrame< double > data,
                                            param.columns_str,
                                            param.verbose );
 
-    if ( E < 1 ) {
-        std::stringstream errMsg;
-        errMsg << " Multiview(): E = " << param.E << " is invalid.\n" ;
-        throw std::runtime_error( errMsg.str() );
-    }
     size_t shift = abs( param.tau ) * ( param.E - 1 );
     
     // Delete data top rows of partial data
@@ -205,7 +222,7 @@ MultiviewValues  Multiview( DataFrame< double > data,
         
     // Adjust param.library and param.prediction vectors of indices
     if ( shift > 0 ) {
-        param.DeleteLibPred( shift );
+        param.DeleteLibPred();
     }
 
     // Get target
@@ -215,13 +232,28 @@ MultiviewValues  Multiview( DataFrame< double > data,
     std::string outputFile = param.predictOutputFile;
     param.predictOutputFile = "";
     
-    // Combinations of possible embedding variables (columns), E at-a-time
+    // Establish the state-space dimension D
+    // default to the number of input columns (not embedded columns)
+    if ( D == 0 ) {
+        D = param.columnNames.size();
+    }
+    if ( D > embedding.NColumns() ) {
+        std::stringstream msg;
+        msg << "WARNING: Multiview(): D = " << D
+            << " exceeds the number of columns in the embedding: "
+            << embedding.NColumns() << ".  D set to "
+            << embedding.NColumns() << std::endl;
+        std::cout << msg.str();
+        
+        D = param.columnNames.size();
+    }
+ 
+    // Combinations of possible embedding variables, D at-a-time
     // Note that these combinations are not zero-offset, i.e.
     // Combination( 3, 2 ) = [(1, 2), (1, 3), (2, 3)]
     // These correspond to column indices +1
-    size_t nVar = param.columnNames.size();
     std::vector< std::vector< size_t > > combos =
-        Combination( nVar * param.E, param.E );
+        Combination( embedding.NColumns(), D );
 
 #ifdef DEBUG_ALL
     std::cout << "Multiview(): " << combos.size() << " combos:\n";
@@ -246,32 +278,50 @@ MultiviewValues  Multiview( DataFrame< double > data,
         std::cout << msg.str();
     }
 
+    // validate number of combinations
+    if ( param.MultiviewEnsemble > combos.size() ) {
+        std::stringstream msg;
+        msg << "WARNING: Multiview(): multiview ensembles "
+            << param.MultiviewEnsemble
+            << " exceeds the number of available combinations: "
+            << combos.size() << ".   Set to " << combos.size() << std::endl;
+        std::cout << msg.str();
+        
+        param.MultiviewEnsemble = combos.size();
+    }
+
     //---------------------------------------------------------------
     // Evaluate variable combinations.
     // Note that this is done within the library itself (in-sample).
     //---------------------------------------------------------------
     // Save a copy of the specified prediction observation rows.
     std::vector<size_t> prediction = param.prediction;
-    
-    // Override the param.prediction for in-sample forecast skill evaluation
-    param.prediction = param.library;
+
+    if ( trainLib ) {
+        // Override param.prediction for in-sample forecast skill evaluation
+        param.prediction = param.library;
+    }
+
+    // This is not a good implementation...
+    // Replace param.E with the number of dimensions, recall embbeded = true
+    param.E = D;
 
     // Create column names for the results DataFrame
-    // One row for each combo: E columns (a combo), rho, MAE, RMSE
+    // One row for each combo: D columns (a combo), rho, MAE, RMSE
     // JP: NOTE DataFrame is based on valarray, so we can't store non
     //     numeric values (column names) in the results DataFrame.
     std::stringstream header;
-    for ( auto i = 1; i <= param.E; i++ ) {
+    for ( auto i = 1; i <= D; i++ ) {
         header << "Col_" << i << " ";
     }
     header << "rho MAE RMSE";
 
-    // Results Data Frame: E columns (a combo), rho, mae, rmse
-    DataFrame<double> combos_rho( combos.size(), param.E + 3, header.str() );
-    
+    // Results Data Frame: D columns (a combo), rho, mae, rmse
+    DataFrame<double> combos_rho( combos.size(), D + 3, header.str() );
+
     // Results vector of DataFrame's with prediction results
     std::vector< DataFrame< double > > combos_prediction( combos.size() );
-        
+
     // Build work queue
     EDM_Multiview::WorkQueue workQ( combos.size() );
 
@@ -282,7 +332,7 @@ MultiviewValues  Multiview( DataFrame< double > data,
 
     unsigned maxThreads = std::thread::hardware_concurrency();
     if ( maxThreads < nThreads ) { nThreads = maxThreads; }
-    
+
     // thread container
     std::vector< std::thread > threads;
     for ( unsigned i = 0; i < nThreads; ++i ) {
@@ -295,7 +345,7 @@ MultiviewValues  Multiview( DataFrame< double > data,
                                         std::ref( combos_rho ),
                                         std::ref( combos_prediction ) ) );
     }
-    
+
     // join threads
     for ( auto &thrd : threads ) {
         thrd.join();
@@ -317,7 +367,7 @@ MultiviewValues  Multiview( DataFrame< double > data,
     }
 
     //-----------------------------------------------------------------
-    // Rank in-sample (library) forecasts
+    // Rank forecasts. If trainLib true these are in-sample (library)
     //-----------------------------------------------------------------
     // Make pairs of row indices and rho
     std::valarray< double > rho = combos_rho.VectorColumnName( "rho" );
@@ -347,15 +397,18 @@ MultiviewValues  Multiview( DataFrame< double > data,
 #endif
 
     // ---------------------------------------------------------------
-    // Perform predictions with the top library multiview embeddings
+    // Perform predictions with the top multiview embeddings
     // ---------------------------------------------------------------
-    // Reset the user specified prediction vector
-    param.prediction = prediction;
+    if ( trainLib ) {
+        // Reset the user specified prediction vector
+        param.prediction = prediction;
+    }
 
     // Get top param.MultiviewEnsemble combos
+    size_t nEnsemble = std::min( (int) combo_sort.size(),
+                                 param.MultiviewEnsemble );
     std::vector< std::pair< double, int > >
-        combo_best( combo_sort.begin(),
-                    combo_sort.begin() + param.MultiviewEnsemble );
+        combo_best( combo_sort.begin(), combo_sort.begin() + nEnsemble );
    
 #ifdef DEBUG_ALL
     std::cout << "Multiview(): Best combos:\n";
@@ -377,56 +430,60 @@ MultiviewValues  Multiview( DataFrame< double > data,
         combos_best[ i ] = this_combo;
     }
     
-    // Results Data Frame: E columns (a combo), and rho mae rmse
+    // Results Data Frame: D columns (a combo), and rho mae rmse
     DataFrame<double> combos_rho_pred( param.MultiviewEnsemble,
-                                       param.E + 3, header.str() );
+                                       D + 3, header.str() );
     
     // Results vector of DataFrame's with prediction results
     // Used to compute the multiview ensemble average prediction
     std::vector< DataFrame< double > >
         combos_rho_prediction( param.MultiviewEnsemble );
 
-    // Build work queue
-    EDM_Multiview::WorkQueue workQ_pred( param.MultiviewEnsemble );
-
-    // Insert combos index into work queue
-    for ( auto i = 0; i < param.MultiviewEnsemble; i++ ) {
-        workQ_pred[ i ] = i;
-    }
-
-    // thread container
-    std::vector< std::thread > threads_pred;
-    for ( unsigned i = 0; i < nThreads; ++i ) {
-        threads_pred.push_back( std::thread( EvalComboThread,
-                                             param,
-                                             workQ_pred,
-                                             combos_best,
-                                             std::ref( embedding ),
-                                             std::ref( targetVec ),
-                                             std::ref( combos_rho_pred ),
-                                             std::ref( combos_rho_prediction)));
-    }
+    //--------------------------------------------------------------------
+    // If trainLib false, no need to compute these projections
+    //--------------------------------------------------------------------
+    if ( trainLib ) {
+        // Build work queue
+        EDM_Multiview::WorkQueue workQ_pred( param.MultiviewEnsemble );
     
-    // join threads
-    for ( auto &thrd : threads_pred ) {
-        thrd.join();
-    }
-    
-    // If thread threw exception, get from queue and rethrow
-    if ( not EDM_Multiview::exceptionQ.empty() ) {
-        std::lock_guard<std::mutex> lck( EDM_Multiview::q_mtx );
-
-        // Take the first exception in the queue
-        std::exception_ptr exceptionPtr = EDM_Multiview::exceptionQ.front();
-
-        // Unroll all other exception from the thread/loops
-        while( not EDM_Multiview::exceptionQ.empty() ) {
-            // JP When do these exception_ptr get deleted? Is it a leak?
-            EDM_Multiview::exceptionQ.pop();
+        // Insert combos index into work queue
+        for ( auto i = 0; i < param.MultiviewEnsemble; i++ ) {
+            workQ_pred[ i ] = i;
+        }
+        
+        // thread container
+        std::vector< std::thread > threads_pred;
+        for ( unsigned i = 0; i < nThreads; ++i ) {
+            threads_pred.push_back(
+                std::thread( EvalComboThread,
+                             param,
+                             workQ_pred,
+                             combos_best,
+                             std::ref( embedding ),
+                             std::ref( targetVec ),
+                             std::ref( combos_rho_pred ),
+                             std::ref( combos_rho_prediction) ) );
+        }
+        
+        // join threads
+        for ( auto &thrd : threads_pred ) {
+            thrd.join();
         }
-        std::rethrow_exception( exceptionPtr );
-    }
     
+        // If thread threw exception, get from queue and rethrow
+        if ( not EDM_Multiview::exceptionQ.empty() ) {
+            std::lock_guard<std::mutex> lck( EDM_Multiview::q_mtx );
+            
+            // Take the first exception in the queue
+            std::exception_ptr exceptionPtr = EDM_Multiview::exceptionQ.front();
+            
+            // Unroll all other exception from the thread/loops
+            while( not EDM_Multiview::exceptionQ.empty() ) {
+                // JP When do these exception_ptr get deleted? Is it a leak?
+                EDM_Multiview::exceptionQ.pop();
+            }
+            std::rethrow_exception( exceptionPtr );
+        }
 #ifdef DEBUG_ALL
     for ( auto cpi =  combos_rho_prediction.begin();
                cpi != combos_rho_prediction.end(); ++cpi ) {
@@ -434,6 +491,21 @@ MultiviewValues  Multiview( DataFrame< double > data,
     }
     std::cout << combos_rho_pred;
 #endif
+    } // if ( trainLib )
+    //-------------------------------------------------------------------
+    // else: trainLib = false
+    //   Projections were initally made with lib != pred
+    //-------------------------------------------------------------------
+    else {
+        // Insert top prediction results into combos_rho_pred
+        // Insert top predictions into combos_rho_prediction
+        int row;
+        for ( size_t row_i = 0; row_i < param.MultiviewEnsemble; row_i++ ) {
+            row = combo_sort[ row_i ].second; // row index of best rho
+            combos_rho_pred.WriteRow( row_i, combos_rho.Row( row ) );
+            combos_rho_prediction[ row_i ] = combos_prediction[ row ];
+        }
+    }
     
     //----------------------------------------------------------
     // Compute Multiview averaged prediction
@@ -471,7 +543,8 @@ MultiviewValues  Multiview( DataFrame< double > data,
     DataFrame< double > Prediction( Predictions.size(), 2,
                                     "Observations  Predictions" );
     // Output time vector
-    std::vector< std::string > predTime( param.prediction.size() + param.Tp );
+    std::vector< std::string > predTime( param.prediction.size() +
+                                         abs( param.Tp ) );
     
     FillTimes( param, data.Time(), std::ref( predTime ) );
     
@@ -552,7 +625,7 @@ void EvalComboThread( Parameters                            param,
 
         // Select combo columns from the data
         DataFrame<double> comboData =
-            embedding.DataFrameFromColumnIndex(combo_cols);
+            embedding.DataFrameFromColumnIndex( combo_cols );
 
         // Compute neighbors on comboData
         Neighbors neighbors = FindNeighbors( comboData, param );
@@ -562,6 +635,7 @@ void EvalComboThread( Parameters                            param,
                                             targetVec, neighbors );
 
         // combo prediction
+        // This is an embedded = true, E = D columns prediction
         DataFrame<double> S = SimplexProjection( param, embedNN );
 
         // Write combo prediction DataFrame
@@ -570,7 +644,7 @@ void EvalComboThread( Parameters                            param,
         // Evaluate combo prediction
         VectorError ve = ComputeError( S.VectorColumnName( "Observations" ),
                                        S.VectorColumnName( "Predictions"  ) );
-
+        
 #ifdef DEBUG_ALL
         {
             std::lock_guard<std::mutex> lck( EDM_Multiview::mtx );
@@ -583,7 +657,7 @@ void EvalComboThread( Parameters                            param,
 #endif
 
         // Write combo and rho to the Data Frame
-        // E columns (a combo), E column names, rho, MAE, RMSE
+        // D columns (a combo), rho, MAE, RMSE
         std::valarray< double > combo_row( combo.size() + 3 );
         for ( auto i = 0; i < combo.size(); i++ ) {
             combo_row[ i ] = combo[ i ];
