@@ -3,19 +3,28 @@
 
 #include <cmath>
 #include <iterator>
+#include <functional>
 
 #include "Common.h"
 #include "Parameter.h"
 
-// Return structure of FindNeighbors()
+// Return structure of FindNeighbors() & Distances()
 struct Neighbors {
     DataFrame<size_t> neighbors;
     DataFrame<double> distances;
+
+    bool anyTies; // Are there ties?
+    std::vector< bool > ties; // true : false for each prediction row
+    std::vector< std::vector< std::pair< double, size_t > > > tiePairs;
+    
     Neighbors();
     ~Neighbors();
 };
 
 // Prototypes
+Neighbors Distances( const DataFrame< double > &dataBlock,
+                           Parameters           param );
+
 Neighbors FindNeighbors( DataFrame<double> dataFrame,
                          Parameters        parameters );
 
