@@ -17,7 +17,7 @@ Parameters::Parameters(
     int         Tp,
     int         knn,
     int         tau,
-    float       theta,
+    double      theta,
     int         exclusionRadius,
 
     std::string columns_str,
@@ -31,9 +31,9 @@ Parameters::Parameters(
     std::string blockFile,
     std::string derivatives_str,
     
-    float       svdSig,
-    float       tikhonov,
-    float       elasticNet,
+    double      svdSig,
+    double      tikhonov,
+    double      elasticNet,
     
     int         multi,
     std::string libSizes_str,
@@ -84,7 +84,7 @@ Parameters::Parameters(
 
     // Set validated flag and instantiate Version
     validated        ( false ),
-    version          ( 1, 2, 1, "2020-02-05" )
+    version          ( 1, 3, 8, "2020-05-23" )
 {
     // Constructor code
     if ( method != Method::None ) {
@@ -121,64 +121,108 @@ void Parameters::Validate() {
         throw std::runtime_error( errMsg );
     }
 
-    if ( Tp < 0 ) {
-        std::string errMsg( "Parameters::Validate(): "
-                            "Tp must be positive.\n" );
-        throw std::runtime_error( errMsg );
-    }
-
     //--------------------------------------------------------------
     // Generate library indices: Apply zero-offset
     //--------------------------------------------------------------
     if ( lib_str.size() ) {
+        // Parse lib_str into vector of strings
         std::vector<std::string> lib_vec = SplitString( lib_str, " \t," );
-        if ( lib_vec.size() != 2 ) {
+        if ( lib_vec.size() % 2 != 0 ) {
             std::string errMsg( "Parameters::Validate(): "
-                                "library must be two integers.\n" );
+                                "library must be even number of integers.\n" );
             throw std::runtime_error( errMsg );
         }
-        int lib_start = std::stoi( lib_vec[0] );
-        int lib_end   = std::stoi( lib_vec[1] );
 
-        if ( method == Method::Simplex or method == Method::SMap ) {
-            // Don't check if None, Embed or CCM since default of "1 1" is used.
-            if ( lib_start >= lib_end ) {
-                std::stringstream errMsg;
-                errMsg << "Parameters::Validate(): library start "
-                       << lib_start << " exceeds end " << lib_end << ".\n";
-                throw std::runtime_error( errMsg.str() );
+        // Generate vector of start, stop index pairs
+        std::vector< std::pair< size_t, size_t > > libPairs;
+        for ( size_t i = 0; i < lib_vec.size(); i = i + 2 ) {
+            libPairs.emplace_back( std::make_pair( std::stoi( lib_vec[i] ),
+                                                   std::stoi( lib_vec[i+1] ) ) );
+        }
+
+        size_t nLib = 0; // Count of lib items
+
+        // Get number of lib indices, validate end > start
+        for ( auto thisPair : libPairs ) {
+            size_t lib_start = thisPair.first;
+            size_t lib_end   = thisPair.second;
+            
+            nLib += lib_end - lib_start + 1;
+            
+            // Validate end > stop indices
+            if ( method == Method::Simplex or method == Method::SMap ) {
+                // Don't check if method == None, Embed or CCM since default
+                // of "1 1" is used.
+                if ( lib_start >= lib_end ) {
+                    std::stringstream errMsg;
+                    errMsg << "Parameters::Validate(): library start "
+                           << lib_start << " exceeds end " << lib_end << ".\n";
+                    throw std::runtime_error( errMsg.str() );
+                }
+            }
+        }
+
+        // Create library vector of indices
+        library = std::vector< size_t >( nLib );
+        size_t i = 0;
+        for ( auto thisPair : libPairs ) {
+            for ( size_t li = thisPair.first; li <= thisPair.second; li++ ) {
+                library[ i ] = li - 1; // apply zero-offset
+                i++;
             }
         }
-        
-        library = std::vector<size_t>( lib_end - lib_start + 1 );
-        std::iota ( library.begin(), library.end(), lib_start - 1 );
     }
 
     //--------------------------------------------------------------
     // Generate prediction indices: Apply zero-offset
     //--------------------------------------------------------------
     if ( pred_str.size() ) {
+        // Parse pred_str into vector of strings
         std::vector<std::string> pred_vec = SplitString( pred_str, " \t," );
-        if ( pred_vec.size() != 2 ) {
+        if ( pred_vec.size() % 2 != 0 ) {
             std::string errMsg( "Parameters::Validate(): "
-                                "prediction must be two integers.\n");
+                                "prediction must be even number of integers.\n");
             throw std::runtime_error( errMsg );
         }
-        int pred_start = std::stoi( pred_vec[0] );
-        int pred_end   = std::stoi( pred_vec[1] );
-        
-        if ( method == Method::Simplex or method == Method::SMap ) {
-            // Don't check if None, Embed or CCM since default of "1 1" is used.
-            if ( pred_start >= pred_end ) {
-                std::stringstream errMsg;
-                errMsg << "Parameters::Validate(): prediction start "
-                       << pred_start << " exceeds end " << pred_end << ".\n";
-                throw std::runtime_error( errMsg.str() );
+
+        // Generate vector of start, stop index pairs
+        std::vector< std::pair< size_t, size_t > > predPairs;
+        for ( size_t i = 0; i < pred_vec.size(); i = i + 2 ) {
+            predPairs.emplace_back( std::make_pair( std::stoi( pred_vec[i] ),
+                                                    std::stoi( pred_vec[i+1])));
+        }
+
+        size_t nPred = 0; // Count of pred items
+
+        // Get number of pred indices, validate end > start
+        for ( auto thisPair : predPairs ) {
+            size_t pred_start = thisPair.first;
+            size_t pred_end   = thisPair.second;
+            
+            nPred += pred_end - pred_start + 1;
+            
+            // Validate end > stop indices
+            if ( method == Method::Simplex or method == Method::SMap ) {
+                // Don't check if method == None, Embed or CCM since default
+                // of "1 1" is used.
+                if ( pred_start >= pred_end ) {
+                    std::stringstream errMsg;
+                    errMsg << "Parameters::Validate(): prediction start "
+                           << pred_start << " exceeds end " << pred_end << ".\n";
+                    throw std::runtime_error( errMsg.str() );
+                }
+            }
+        }
+
+        // Create prediction vector of indices
+        prediction = std::vector< size_t >( nPred );
+        size_t i = 0;
+        for ( auto thisPair : predPairs ) {
+            for ( size_t li = thisPair.first; li <= thisPair.second; li++ ) {
+                prediction[ i ] = li - 1; // apply zero-offset
+                i++;
             }
         }
-        
-        prediction = std::vector<size_t>( pred_end - pred_start + 1 );
-        std::iota ( prediction.begin(), prediction.end(), pred_start - 1 );
     }
     
     if ( method == Method::Simplex or method == Method::SMap ) {
@@ -385,7 +429,7 @@ void Parameters::Validate() {
         }
         else {
             // default knn = 0, set knn value
-            knn = library.size() - Tp * (E + 1);
+            knn = library.size() - abs( Tp ) * (E + 1);
             if ( verbose ) {
                 std::stringstream msg;
                 msg << "Parameters::Validate(): Set knn = " << knn
@@ -451,8 +495,9 @@ void Parameters::Validate() {
 // Adjust lib/pred concordant with Embed() removal of tau(E-1)
 // rows, and DeletePartialDataRow()
 //------------------------------------------------------------
-void Parameters::DeleteLibPred( size_t shift ) {
-    
+void Parameters::DeleteLibPred() {
+
+    size_t shift          = abs( tau ) * ( E - 1 );
     size_t library_len    = library.size();
     size_t prediction_len = prediction.size();
 
@@ -474,27 +519,69 @@ void Parameters::DeleteLibPred( size_t shift ) {
                    deleted_lib_elements.end(), library_len - shift );
     }
 
-    // Erase elements of row indices that were deleted
+    // Now that we have the indices that could have been deleted,
+    // check to see if any are in lib and pred
+    bool deleteLibIndex = false;
+    std::vector< size_t >::iterator it;
     for ( auto element  = deleted_lib_elements.begin();
                element != deleted_lib_elements.end(); element++ ) {
-
-        std::vector< size_t >::iterator it;
         it = std::find( library.begin(), library.end(), *element );
-
         if ( it != library.end() ) {
-            library.erase( it );
+            deleteLibIndex = true;
+            break;
         }
     }
-                
+    
+    bool deletePredIndex = false;
     for ( auto element  = deleted_pred_elements.begin();
                element != deleted_pred_elements.end(); element++ ) {
-
-        std::vector< size_t >::iterator it;
-        it = std::find( prediction.begin(),
-                        prediction.end(), *element );
-
+        it = std::find( prediction.begin(), prediction.end(), *element );
         if ( it != prediction.end() ) {
-            prediction.erase( it );
+            deletePredIndex = true;
+            break;
+        }
+    }
+    
+#ifdef DEBUG_ALL
+    std::cout << "DeleteLibPred(): Nrows: " << NRows
+              << " deleteLibIndex: " << deleteLibIndex
+              << " deletePredIndex: " << deletePredIndex << std::endl;
+    std::cout << " deleted_lib_elements: ";
+    for ( auto element  = deleted_lib_elements.begin();
+               element != deleted_lib_elements.end(); element++ ) {
+        std::cout << *element << ", ";
+    } std::cout << std::endl << " deleted_pred_elements: ";
+    for ( auto element  = deleted_pred_elements.begin();
+               element != deleted_pred_elements.end(); element++ ) {
+        std::cout << *element << ", ";
+    } std::cout << std::endl;
+#endif
+        
+    // Erase elements of row indices that were deleted
+    if ( deleteLibIndex ) {
+        for ( auto element  = deleted_lib_elements.begin();
+              element != deleted_lib_elements.end(); element++ ) {
+            
+            std::vector< size_t >::iterator it;
+            it = std::find( library.begin(), library.end(), *element );
+            
+            if ( it != library.end() ) {
+                library.erase( it );
+            }
+        }
+    }
+
+    if ( deletePredIndex ) {
+        for ( auto element  = deleted_pred_elements.begin();
+              element != deleted_pred_elements.end(); element++ ) {
+            
+            std::vector< size_t >::iterator it;
+            it = std::find( prediction.begin(),
+                            prediction.end(), *element );
+            
+            if ( it != prediction.end() ) {
+                prediction.erase( it );
+            }
         }
     }
             
@@ -510,6 +597,8 @@ void Parameters::DeleteLibPred( size_t shift ) {
         }
     }
     // tau > 0  : Forward shifting: no adjustment needed from origin
+
+    return;
 }
 
 //------------------------------------------------------------------
