@@ -6,19 +6,16 @@
 #include "AuxFunc.h"
 
 // forward declarations
-std::valarray < double > SVD( DataFrame    < double > A,
-                              std::valarray< double > B );
-
 std::valarray< double >  Lapack_SVD( int     m,
                                      int     n,
                                      double *a,
                                      double *b,
                                      double  rcond );
 
-//----------------------------------------------------------------
-// Overload 1: Explicit data file path/name
-//   Implemented as a wrapper to API Overload 2:
-//----------------------------------------------------------------
+//---------------------------------------------------------------------
+// Overload 1: Explicit data file path/name with internal SVD (LAPACK)
+//   Implemented as a wrapper to API overload 2 -> 4
+//---------------------------------------------------------------------
 SMapValues SMap( std::string pathIn,
                  std::string dataFile,
                  std::string pathOut,
@@ -41,7 +38,8 @@ SMapValues SMap( std::string pathIn,
 {
     // DataFrame constructor loads data
     DataFrame< double > dataFrameIn( pathIn, dataFile );
-    
+
+    // Call overload 2 with the dataFrameIn
     SMapValues SMapOutput = SMap( dataFrameIn, pathOut, predictFile,
                                   lib, pred, E, Tp, knn, tau, theta,
                                   exclusionRadius,
@@ -51,7 +49,8 @@ SMapValues SMap( std::string pathIn,
 }
 
 //----------------------------------------------------------------
-// Overload 2: DataFrame provided
+// Overload 2: DataFrame provided with internal SVD (LAPACK)
+//    Implemented as a wrapper to API overload 4
 //----------------------------------------------------------------
 SMapValues SMap( DataFrame< double > &data,
                  std::string pathOut,
@@ -70,6 +69,79 @@ SMapValues SMap( DataFrame< double > &data,
                  std::string derivatives,
                  bool        embedded,
                  bool        const_predict,
+                 bool        verbose ) {
+
+    // Call overload 4 with default SVD function (below)
+    SMapValues SMapOutput = SMap( data, pathOut, predictFile,
+                                  lib, pred, E, Tp, knn, tau, theta, 
+                                  exclusionRadius,
+                                  columns, target, smapFile, derivatives,
+                                  &SVD,
+                                  embedded, const_predict, verbose);
+
+    return SMapOutput;
+}
+
+//----------------------------------------------------------------
+// Overload 3: Explicit data file path/name and solver
+//   Implemented as a wrapper to API overload 4
+//----------------------------------------------------------------
+SMapValues SMap( std::string pathIn,
+                 std::string dataFile,
+                 std::string pathOut,
+                 std::string predictFile,
+                 std::string lib,
+                 std::string pred,
+                 int         E,
+                 int         Tp,
+                 int         knn,
+                 int         tau,
+                 double      theta,
+                 int         exclusionRadius,
+                 std::string columns,
+                 std::string target,
+                 std::string smapFile,
+                 std::string derivatives,
+                 std::valarray<double> (*solver) (DataFrame < double >,
+                                                  std::valarray < double > ),
+                 bool        embedded,
+                 bool        const_predict,
+                 bool        verbose )
+{
+    // DataFrame constructor loads data
+    DataFrame< double > dataFrameIn( pathIn, dataFile );
+    
+    // Call overload 4 with dataFrameIn and solver object
+    SMapValues SMapOutput = SMap( dataFrameIn, pathOut, predictFile,
+                                  lib, pred, E, Tp, knn, tau, theta,
+                                  exclusionRadius,
+                                  columns, target, smapFile, derivatives, 
+                                  solver, embedded, const_predict, verbose );
+    return SMapOutput;
+}
+
+//----------------------------------------------------------------
+// Overload 4: Solver & DataFrame provided
+//----------------------------------------------------------------
+SMapValues SMap( DataFrame< double > &data,
+                 std::string pathOut,
+                 std::string predictFile,
+                 std::string lib,
+                 std::string pred,
+                 int         E,
+                 int         Tp,
+                 int         knn,
+                 int         tau,
+                 double      theta,
+                 int         exclusionRadius,
+                 std::string columns,
+                 std::string target,
+                 std::string smapFile,
+                 std::string derivatives,
+                 std::valarray<double> (*solver) (DataFrame < double >,
+                                                  std::valarray < double > ),
+                 bool        embedded,
+                 bool        const_predict,
                  bool        verbose )
 {
 
@@ -120,6 +192,7 @@ SMapValues SMap( DataFrame< double > &data,
     }
     
     std::valarray< double > predictions = std::valarray< double >( N_row );
+    std::valarray< double > variance    = std::valarray< double >( N_row );
 
     // Init coefficients to NAN ?
     DataFrame< double > coefficients = DataFrame< double >( N_row,
@@ -148,7 +221,7 @@ SMapValues SMap( DataFrame< double > &data,
 
         // Populate matrix A (exp weighted future prediction), and
         // vector B (target BC's) for this row (observation).
-        size_t lib_row;
+        int    lib_row;
         size_t lib_row_base;
         
         for ( size_t k = 0; k < param.knn; k++ ) {
@@ -167,6 +240,9 @@ SMapValues SMap( DataFrame< double > &data,
                 // Use the neighbor at the 'base' of the trajectory
                 B[ k ] = target_vec[ lib_row_base ];
             }
+            else if ( lib_row < 0 ) {
+                B[ k ] = target_vec[ 0 ];
+            }
             else {
                 B[ k ] = target_vec[ lib_row ];
             }
@@ -190,7 +266,7 @@ SMapValues SMap( DataFrame< double > &data,
         B = w * B; // Weighted target vector
 
         // Estimate linear mapping of predictions A onto target B
-        std::valarray < double > C = SVD( A, B );
+        std::valarray < double > C = solver( A, B );
 
         // Prediction is local linear projection
         double prediction = C[ 0 ]; // C[ 0 ] is the bias term
@@ -203,6 +279,11 @@ SMapValues SMap( DataFrame< double > &data,
         predictions[ row ] = prediction;
         coefficients.WriteRow( row, C );
 
+        // "Variance" estimate assuming weights are probabilities
+        std::valarray< double > deltaSqr =
+            std::pow( B - predictions[ row ], 2);
+        variance[ row ] = ( w * deltaSqr ).sum() / w.sum();
+        
     } // for ( row = 0; row < predict_N_row; row++ )
 
     // non "predictions" X(t+1) = X(t) if const_predict specified
@@ -226,6 +307,7 @@ SMapValues SMap( DataFrame< double > &data,
     DataFrame<double> dataOut = FormatOutput( param,
                                               predictions,
                                               const_predictions,
+                                              variance,
                                               target_vec,
                                               dataIn.Time(),
                                               dataIn.TimeName() );
@@ -252,11 +334,17 @@ SMapValues SMap( DataFrame< double > &data,
 
     // coefficients have N_row's; coefOut has N_row + Tp
     // Create coefficient column vector with Tp nan rows at the
-    // beginning of coefOut as in FormatOutput()
+    // beginning/end of coefOut as in FormatOutput()
     std::valarray<double> coefColumnVec( NAN, dataOut.NRows() );
     
     // Copy coefficients vectors into coefOut
-    std::slice coef_i = std::slice( param.Tp, N_row, 1 );
+    std::slice coef_i;
+    if ( param.Tp > -1 ) {
+        coef_i = std::slice( param.Tp, N_row, 1 );
+    }
+    else {
+        coef_i = std::slice( 0, N_row + param.Tp, 1 );
+    }
     for ( size_t col = 0; col < coefOut.NColumns(); col++ ) {
         coefColumnVec[ coef_i ] = coefficients.Column( col );
         coefOut.WriteColumn( col, coefColumnVec );
@@ -427,9 +515,9 @@ std::valarray< double > Lapack_SVD( int     m, // number of rows in matrix
     // Copy solution vector in b to C
     std::valarray< double > C( b, N_SingularValues );
 
-    delete s;
-    delete work;
-    delete iwork;
+    delete[] s;
+    delete[] work;
+    delete[] iwork;
     
     return C;
 }
